// StackingMovingAverageCrossover.mq4

#define STRING_EMPTY       ""
#define MAX_DIGITS         8

// Expert Advisor States
#define STATE_UNDEFINED    -1
#define STATE_BUY          0
#define STATE_SELL         1

extern int TimeFrame = 0;

extern int FastMovingAveragePeriod = 3;
extern int FastMovingAverageMethod = 0;
extern int FastMovingAverageApplyToPrice = 0;

extern int SlowMovingAveragePeriod = 20;
extern int SlowMovingAverageMethod = 0;
extern int SlowMovingAverageApplyToPrice = 0;

extern int StackSize = 5;
extern int DistanceApart = 50;

extern double Risk = 0.02;
extern double LotSize = 0.1;

extern int StopLoss = 0;
extern int TrailingStopLossStep = 0;
extern int TakeProfit = 0;
extern bool StealthMode = true;

extern int Slippage = 0;
extern int MaxAttempts = 1;

extern bool ReverseSignals = true;

extern int MagicNumber = 0;

int FastPeriod;
int FastMethod;
int FastPrice;

int SlowPeriod;
int SlowMethod;
int SlowPrice;

int ExpertState = STATE_UNDEFINED;
double ExpertStopLoss = 0;
double ExpertTakeProfit = 0;

void init()
{
   MathSrand(GetTickCount());
   
   switch (TimeFrame)
   {
      case 0:
         TimeFrame = Period();
         break;
      case PERIOD_M1:
      case PERIOD_M5:
      case PERIOD_M15:
      case PERIOD_M30:
      case PERIOD_H1:
      case PERIOD_H4:
      case PERIOD_D1:
      case PERIOD_W1:
      case PERIOD_MN1:
         // OK
         break;
      default:
         TimeFrame = PERIOD_H1;
         break;
   }
   
   FastPeriod = FastMovingAveragePeriod;
   FastMethod = FastMovingAverageMethod;
   FastPrice = FastMovingAverageApplyToPrice;
   
   switch (FastMethod)
   {
      case MODE_SMA:
      case MODE_EMA:
      case MODE_SMMA:
      case MODE_LWMA:
         break;
      default:
         FastMethod = MODE_SMA;
         break;
   }
   
   switch (FastPrice)
   {
      case PRICE_CLOSE:
      case PRICE_OPEN:
      case PRICE_HIGH:
      case PRICE_LOW:
      case PRICE_MEDIAN:
      case PRICE_TYPICAL:
      case PRICE_WEIGHTED:
         break;
      default:
         FastPrice = PRICE_CLOSE;
         break; 
   }
   
   SlowPeriod = SlowMovingAveragePeriod;
   SlowMethod = SlowMovingAverageMethod;
   SlowPrice = SlowMovingAverageApplyToPrice;
   
   switch (SlowMethod)
   {
      case MODE_SMA:
      case MODE_EMA:
      case MODE_SMMA:
      case MODE_LWMA:
         break;
      default:
         SlowMethod = MODE_SMA;
         break;
   }
   
   switch (SlowPrice)
   {
      case PRICE_CLOSE:
      case PRICE_OPEN:
      case PRICE_HIGH:
      case PRICE_LOW:
      case PRICE_MEDIAN:
      case PRICE_TYPICAL:
      case PRICE_WEIGHTED:
         break;
      default:
         SlowPrice = PRICE_CLOSE;
         break; 
   }
   
   if (MagicNumber == 0)
   {
      MagicNumber = generate_magic_number();
      Print("Generated MagicNumber = ", MagicNumber);
   }
}

void start()
{
   double f0, f1, f2; // fast moving average values
   double s0, s1, s2; // slow moving average values
   double d0, d1, d2; // signs of difference (fast - slow)
   double e0, e1;     // established differences
   
   f0 = iMA(Symbol(), TimeFrame, FastPeriod, 0, FastMethod, FastPrice, 0);
   f1 = iMA(Symbol(), TimeFrame, FastPeriod, 0, FastMethod, FastPrice, 1);
   f2 = iMA(Symbol(), TimeFrame, FastPeriod, 0, FastMethod, FastPrice, 2);
   
   s0 = iMA(Symbol(), TimeFrame, SlowPeriod, 0, SlowMethod, SlowPrice, 0);
   s1 = iMA(Symbol(), TimeFrame, SlowPeriod, 0, SlowMethod, SlowPrice, 1);
   s2 = iMA(Symbol(), TimeFrame, SlowPeriod, 0, SlowMethod, SlowPrice, 2);
   
   d0 = compare_doubles(f0, s0, Digits);
   d1 = compare_doubles(f1, s1, Digits);
   d2 = compare_doubles(f2, s2, Digits);
   /*
   if ((FastPrice == PRICE_OPEN) && (SlowPrice == PRICE_OPEN))
   {
      e0 = d0;
      e1 = d1;
   }
   else
   {
      e0 = d1;
      e1 = d2;
   }
   */
   e0 = d0;
   e1 = d1;
   
   int buyState, sellState;
   if (ReverseSignals)
   {
      buyState = STATE_SELL;
      sellState = STATE_BUY;
   }
   else
   {// as usual
      buyState = STATE_BUY;
      sellState = STATE_SELL;   
   }
   
   int currentState = STATE_UNDEFINED;
   if (e0 > 0)
   {
      currentState = buyState;
   }
   else if (e0 < 0)
   {
      currentState = sellState;
   }
   else
   {
      if (e1 > 0)
      {
         currentState = buyState;
      }
      else if (e1 < 0)
      {
         currentState = sellState;
      }
      else
      {
         currentState = ExpertState;
      }
   }
   
   if (ExpertState != currentState)
   {
      ExpertState = currentState;
      
      close_orders();
      
      open_orders();
   }
   
   if ((StopLoss > 0) && (TrailingStopLossStep > 0))
   {
      trail();
   }
   
   if (StealthMode)
   {
      stealth_close();
   }
}

void deinit()
{
}

void close_orders()
{
   int i, j;
   
   for (i = OrdersTotal(); i > 0; i--)
   {
      if (OrderSelect(i - 1, SELECT_BY_POS))
      {
         if (OrderMagicNumber() == MagicNumber)
         {
            if (is_market(OrderType()))
            {
               for (j = 0; j < MaxAttempts; j++)
               {
                  double price;
                  if (OrderType() == OP_BUY)
                  {
                     price = MarketInfo(Symbol(), MODE_BID);
                  }
                  else // if (OrderType() == OP_SELL)
                  {
                     price = MarketInfo(Symbol(), MODE_ASK);
                  }
                  if (OrderClose(OrderTicket(), OrderLots(), price, Slippage))
                  {
                     break;
                  }
                  else
                  {
                     Print("WARNING: CloseOrder returned FALSE for #", OrderTicket());
                  }
               }
            }
            else
            {
               for (j = 0; j < MaxAttempts; j++)
               {
                  if (OrderDelete(OrderTicket()))
                  {
                     break;
                  }
                  else
                  {
                     Print("WARNING: OrderDelete returned FALSE for #", OrderTicket());
                  }
               }
            }
         }
      }
      else
      {
         Print("WARNING: OrderSelect returned FALSE for i = ", i);
      }
   }
}

void open_orders()
{
   int i, j;
   int ticket;
   int cmd;
   double price;
   
   if (ExpertState == STATE_BUY)
   {
      cmd = OP_BUY;
      price = MarketInfo(Symbol(), MODE_ASK);
      
      if (StopLoss > 0)
      {
         ExpertStopLoss = price - StopLoss * Point;
      }
      
      if (TakeProfit > 0)
      {
         ExpertTakeProfit = price + TakeProfit * Point;
      }
   }
   else // if (ExpertState == STATE_SELL)
   {
      cmd = OP_SELL;
      price = MarketInfo(Symbol(), MODE_BID);
      
      if (StopLoss > 0)
      {
         ExpertStopLoss = price + StopLoss * Point;
      }
      
      if (TakeProfit > 0)
      {
         ExpertTakeProfit = price - TakeProfit * Point;
      }
   }
   
   double stopLoss, takeProfit;
   if (StealthMode)
   {
      stopLoss = 0;
      takeProfit = 0;
   }
   else
   {
      stopLoss = ExpertStopLoss;
      takeProfit = ExpertTakeProfit;
   }
   
   for (i = 0; i < StackSize; i++)
   {
      double lotSize;
      if (Risk > 0)
      {
         double minLot = MarketInfo(Symbol(), MODE_MINLOT);
         lotSize = AccountFreeMargin() * Risk * AccountLeverage() / MarketInfo(Symbol(), MODE_LOTSIZE);
         
         if (compare_doubles(lotSize, minLot) >= 0)
         {
            double lotStep = MarketInfo(Symbol(), MODE_LOTSTEP);
            double remainder = MathMod(lotSize - minLot, lotStep);
            
            lotSize = NormalizeDouble(lotSize - remainder, get_digits(lotStep));
            
            Print("Calculated lot size (Risk = ", Risk * 100, "%) = ", lotSize);
         }
         else
         {
            Print("WARNING: Calculated lot size (Risk = ", Risk * 100, "%) = ", lotSize, " is less than minimal lot = ", minLot);
            break;
         }
      }
      else
      {
         lotSize = LotSize;
      }
      
      for (j = 0; j < MaxAttempts; j++)
      {
         ticket = OrderSend(Symbol(), cmd, lotSize, price, Slippage, stopLoss, takeProfit, NULL, MagicNumber);
         if (ticket > 0)
         {
            break;
         }
         else
         {
            Print("WARNING: OrderSend failed, GetLastError returned ", GetLastError());
         }
      }
      
      if (ExpertState == STATE_BUY)
      {
         price += DistanceApart * Point;
      }
      else
      {
         price -= DistanceApart * Point;
      }
      
      if (i == 0)
      {
         if (ExpertState == STATE_BUY)
         {
            cmd = OP_BUYSTOP;
         }
         else
         {
            cmd = OP_SELLSTOP;
         }
      }
   }
}

void trail()
{
   double distance;
   if (ExpertState == STATE_BUY)
   {
      distance = MarketInfo(Symbol(), MODE_ASK) - ExpertStopLoss;
   }
   else // if (ExpertState == STATE_SELL)
   {
      distance = ExpertStopLoss - MarketInfo(Symbol(), MODE_BID);
   }
   
   if (MathRound(distance / Point) >= StopLoss + TrailingStopLossStep)
   {
      if (ExpertState == STATE_BUY)
      {
         ExpertStopLoss += TrailingStopLossStep * Point;
      }
      else
      {
         ExpertStopLoss -= TrailingStopLossStep * Point;
      }
      
      if (StealthMode == false)
      {
         for (int i = OrdersTotal(); i > 0; i--)
         {
            if (OrderSelect(i - 1, SELECT_BY_POS))
            {
               if (OrderMagicNumber() == MagicNumber)
               {
                  if (is_market(OrderType()))
                  {
                     for (int j = 0; j < MaxAttempts; j++)
                     {
                        if (OrderModify(OrderTicket(), OrderOpenPrice(), ExpertStopLoss, OrderTakeProfit(), 0))
                        {
                           break;
                        }
                        else
                        {
                           Print("WARNING: OrderModify returned FALSE, GetLastError returned ", GetLastError());
                        }
                     }
                  }
               }
            }
            else
            {
               Print("WARNING: OrderSelect returned FALSE for order number = ", i - 1);
            }
         }
      }
   }
}

void stealth_close()
{
   for (int i = OrdersTotal(); i > 0; i--)
   {
      if (OrderSelect(i - 1, SELECT_BY_POS))
      {
         if (OrderMagicNumber() == MagicNumber)
         {
            if (is_market(OrderType()))
            {
               double ask = MarketInfo(Symbol(), MODE_ASK);
               double bid = MarketInfo(Symbol(), MODE_BID);
               double closePrice = 0;
               if (OrderType() == OP_BUY)
               {
                  if (((StopLoss > 0) && (bid <= ExpertStopLoss)) || ((TakeProfit > 0) && (ask >= ExpertTakeProfit)))
                  {
                     closePrice = bid;
                  }
               }
               else
               {
                  if (((TakeProfit > 0) && (bid <= ExpertTakeProfit)) || ((StopLoss > 0) && (ask >= ExpertStopLoss)))
                  {
                     closePrice = ask;
                  }
               }
               
               if (closePrice > 0)
               {
                  for (int j = 0; j < MaxAttempts; j++)
                  {
                     if (OrderClose(OrderTicket(), OrderLots(), closePrice, Slippage))
                     {
                        break;
                     }
                     else
                     {
                        Print("WARNING: OrderClose returned FALSE, GetLastError returned ", GetLastError());
                     }
                  }
               }
            }
         }
      }
      else
      {
         Print("WARNING: OrderSelect returned FALSE for order number = ", i - 1);
      }
   }
}

int compare_doubles(double a, double b, int digits = MAX_DIGITS)
{
   int result;
   double d = MathRound((a - b) * MathPow(10, digits));
   if (d > 0)
   {
      result = 1;
   }
   else if (d < 0)
   {
      result = -1;
   }
   else
   {
      result = 0;
   }
   return (result);
}

bool is_market(int orderType)
{
   bool result;
   
   switch (orderType)
   {
      case OP_BUY:
      case OP_SELL:
         result = true;
         break;
      default:
         result = false;
         break;
   }
   
   return (result);
}

int generate_magic_number()
{
   return (2147483647 / 32767 * MathRand());
}

int get_digits(double d, int maxDigits = MAX_DIGITS)
{
   int digits = 0;
   while (MathAbs(NormalizeDouble(d - MathRound(d), maxDigits)) > 0)
   {
      digits++;
      d = d * 10;
   }
   return (digits);
}